@@ -7,7 +7,7 @@ import yaml
 
 from steam_sdk.data.DataSettings import DataSettingsSTEAM
 from steam_sdk.builders.BuilderModel import BuilderModel
-from steam_sdk.data.DataAnalysis import DataAnalysis, AnalysisStep
+from steam_sdk.data.DataAnalysis import DataAnalysis, ModifyModel
 from steam_sdk.drivers.DriverLEDET import DriverLEDET
 from steam_sdk.drivers.DriverPSPICE import DriverPSPICE
 from steam_sdk.drivers.DriverPyBBQ import DriverPyBBQ
@@ -185,8 +185,7 @@ class AnalysisSTEAM:
         if verbose:
             print('Defined analysis steps (not in sequential order):')
             for def_step in step_definitions:
-                step = def_step.step
-                print('{}'.format(step.step_name))
+                print(f'{def_step}')
 
         # Print analysis sequence
         if verbose: print('Defined sequence of analysis steps:')
@@ -197,11 +196,8 @@ class AnalysisSTEAM:
         if verbose: print('Analysis started.')
         for s, seq_step in enumerate(self.data_analysis.AnalysisStepSequence):
             if verbose: print('Step {}/{}: {}'.format(s + 1, len(self.data_analysis.AnalysisStepSequence), seq_step))
-            current_step = (i for i, v in enumerate(step_definitions) if step_definitions[i].step.step_name == seq_step)
-            current_step = next(current_step, None)
-            # if verbose: print('Index of current step: {}'.format(current_step))
 
-            step = step_definitions[current_step].step  # this is the object containing the information about the current step
+            step = step_definitions[seq_step]  # this is the object containing the information about the current step
             if step.type == 'MakeModel':
                 self.step_make_model(step, verbose=verbose)
             elif step.type == 'ModifyModel':
@@ -392,8 +388,8 @@ class AnalysisSTEAM:
 
         # Loop through the list of variables to change
         for v, variable_to_change in enumerate(step.variables_to_change):
-            # For each variable to change, make an instance of AnalysisStep and call the step_modify_model() method
-            next_step = AnalysisStep(step={'type': 'ModifyModel'}).step
+            # For each variable to change, make an instance of an ModifyModel step and call the step_modify_model() method
+            next_step = ModifyModel(type='ModifyModel')
             next_step.model_name = step.model_name
             next_step.variable_to_change = variable_to_change
             next_step.variable_value = step.variables_value[v]
@@ -407,7 +403,7 @@ class AnalysisSTEAM:
             next_step.software = step.software
             self.step_modify_model(next_step, verbose=verbose)
         if verbose:
-            print('All variables of step {} were changed.'.format(step.step_name))
+            print('All variables of step {} were changed.'.format(step))
 
 
     def step_run_simulation(self, step, verbose: bool = False):
