@@ -1,6 +1,6 @@
 import numpy as np
 
-from steam_sdk.utils.misc import progressBar
+from steam_sdk.utils.misc import progressBar, find_indices
 
 
 class SelfMutualInductanceCalculation:
@@ -8,7 +8,7 @@ class SelfMutualInductanceCalculation:
     EPS = 1e-12
 
     def __init__(self, x, y, polarities, Ns, Ds, strandToHalfTurn, strandToCoilSection, flag_strandCorrection=0,
-                 flag_sumTurnToTurn=1, flag_writeOutput=0, nameMagnet=''):
+                 flag_sumTurnToTurn=1, flag_writeOutput=0, nameMagnet='', verbose: str = False):
         self.nameMagnet = nameMagnet
         self.x = x
         self.y = y
@@ -19,6 +19,7 @@ class SelfMutualInductanceCalculation:
         self.flag_strandCorrection = flag_strandCorrection
         self.flag_sumTurnToTurn = flag_sumTurnToTurn
         self.flag_writeOutput = flag_writeOutput
+        self.verbose = verbose
 
     def getNameMagnet(self):
         return self.nameMagnet
@@ -67,13 +68,13 @@ class SelfMutualInductanceCalculation:
 
         # Calculate self-mutual inductance between strands
         oneTwentiethStrands = np.round(nStrands / 20)  # This is used only to show progress during the calculation
-        print('Self-mutual inductance between strands - Calculation started')
+        if self.verbose: print('Self-mutual inductance between strands - Calculation started')
         if self.flag_strandCorrection == 0:
-            print('Calculation does not include correction for strand diameter.')
+            if self.verbose: print('Calculation does not include correction for strand diameter.')
             M_strands = np.zeros((nStrands, nStrands))
             # Calculate mutual inductances between strands
             for p1 in range(nStrands):
-                if p1 % oneTwentiethStrands == 0:
+                if self.verbose and p1 % oneTwentiethStrands == 0:
                     progressBar(p1, nStrands, bar_length=20)
                 M_strands[p1, :] = -np.sign(polarities[p1]) / Ns[strandToHalfTurn[p1] - 1] * np.sign(polarities[:]) / \
                                    Ns[strandToHalfTurn[:] - 1] * mu0 / 2 / np.pi * np.log(
@@ -95,22 +96,23 @@ class SelfMutualInductanceCalculation:
         #         idxDiagStrands = np.diag_indices(nStrands)
         #         M_strands[idxDiagStrands]=mu0/np.pi/8/Ns[strandToHalfTurn[:]-1]
 
-        progressBar(1, 1, bar_length=20)
-        print('')
-        print('Self-mutual inductance between strands - Calculation finished')
+        if self.verbose:
+            progressBar(1, 1, bar_length=20)
+            print('')
+            print('Self-mutual inductance between strands - Calculation finished')
         return M_strands
 
     def calculateHalfTurnInductance(self, M_strands, strandToHalfTurn):
-        print('Self-mutual inductance between half-turns - Calculation started')
+        if self.verbose: print('Self-mutual inductance between half-turns - Calculation started')
 
         # Calculate self-mutual inductance between half-turns
         M_halfTurns = self.sumMatrixElements(M_strands, strandToHalfTurn)
 
-        print('Self-mutual inductance between half-turns - Calculation finished')
+        if self.verbose: print('Self-mutual inductance between half-turns - Calculation finished')
         return M_halfTurns
 
     def calculateTurnInductance(self, M_halfTurns):
-        print('Self-mutual inductance between turns - Calculation started')
+        if self.verbose: print('Self-mutual inductance between turns - Calculation started')
 
         # Calculate number of turns
         # THIS ASSUMES THAT THE SECOND HALF OF THE HALF-TURNS ARE THE RETURN LINES OF THE FIRST HALF
@@ -120,7 +122,7 @@ class SelfMutualInductanceCalculation:
         # Calculate self-mutual inductance between turns
         M_turns = self.sumMatrixElements(M_halfTurns, halfTurnToTurn)
 
-        print('Self-mutual inductance between turns - Calculation finished')
+        if self.verbose: print('Self-mutual inductance between turns - Calculation finished')
         return M_turns
 
     def calculateCoilSectionInductance(self, M_strands, strandToCoilSection):
@@ -133,19 +135,51 @@ class SelfMutualInductanceCalculation:
         L_mag0 = np.sum(M_strands)
         return L_mag0
 
-    def sumMatrixElements(self, M, indexToGroup):
+    def sumMatrixElements_OLD(self, M, indexToGroup):
         nGroups = np.max(indexToGroup)
 
         # Calculate self-mutual inductance between half-turns
         M_sum = np.empty((nGroups, nGroups))
         for g1 in range(nGroups):
-            idxG1 = np.where(indexToGroup == g1 + 1)
+            idxG1 = np.where(indexToGroup == g1 + 1)[0]
             for g2 in range(nGroups):
-                idxG2 = np.where(indexToGroup == g2 + 1)
-                M_sum[g1, g2] = np.sum(M[idxG1[0][:, None], idxG2[0]])
+                idxG2 = np.where(indexToGroup == g2 + 1)[0]
+                M_sum[g1, g2] = np.sum(M[idxG1[:, None], idxG2])
 
         return M_sum
 
+    def sumMatrixElements(self, M, indexToGroup):
+        nGroups = np.max(indexToGroup)
+
+        # Pre-calculate indices
+        idxG = []  # initialize an empty list taht will be filled with np.arrays
+        for g in range(nGroups):
+            idxG.append(np.where(indexToGroup == g + 1)[0])  # each element of the list is an array containing the indices of the elements that belong to group g
+
+        # Calculate self-mutual inductance between half-turns
+        M_sum = np.empty((nGroups, nGroups))
+        for g1 in range(nGroups):
+            for g2 in range(nGroups):
+                M_sum[g1, g2] = np.sum(M[idxG[g1][:, None], idxG[g2]])
+
+        return M_sum
+
+    # # It works but it is extremely slow
+    # def sumMatrixElements(self, M, indexToGroup):
+    #     nGroups = np.max(indexToGroup)
+    #
+    #     # Calculate self-mutual inductance between half-turns
+    #     M_sum = np.empty((nGroups, nGroups))
+    #     for g1 in range(nGroups):
+    #         idxG1 = find_indices(indexToGroup, lambda s: s == g1 + 1)  # indices of the items belonging to group g1
+    #         idxG1_mod = np.array(idxG1)
+    #         for g2 in range(nGroups):
+    #             idxG2 = find_indices(indexToGroup, lambda s: s == g2 + 1)  # indices of the items belonging to group g2
+    #             idxG2_mod = np.array(idxG2)[:,None]
+    #             M_sum[g1, g2] = np.sum(M[idxG1_mod, idxG2_mod])
+    #
+    #     return M_sum
+
     def calculateInductance(self, x, y, polarities, Ns, Ds, strandToHalfTurn, strandToCoilSection,
                             flag_strandCorrection=0):
         # Check inputs
