@@ -114,8 +114,7 @@ class Viewer:
                 raise Exception(f'Configuration file {configuration_file} missing.')
 
             # Read configuration file (note: different events can use different configuration files)
-            # dict_current_configuration_file = yaml_as_dict(configuration_file)
-            temp_dict_here = yaml.safe_load(open(configuration_file))  # TODO explore alternative way to read this dictionary
+            temp_dict_here = yaml.safe_load(open(configuration_file))
             current_configuration_file = DataSignal(**temp_dict_here)
             if not configuration_name in current_configuration_file.ConfigurationList:
                 raise Exception(f'Row {t}: Event: {event_label}. Configuration {configuration_name} is not present in file {configuration_file}.')
